<?php

/*

    FASDIM: Fast And Simple DeIdentification Method
    Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/

print("
STEP 01
This program lists your DOC files (doc_nonano).
A MySQL table will be created. A FASDIM database must first exist.
The DOC files must already exist in the corresponding directory.
DOCX files are not taken into account.

") ;


$liste_param=array('doc_nonano', 'txt_nonano', 'txt_preano', 'txt_ano',
	'database_dsn', 'database_user', 'database_password',
	'table_docnonano2txtnonano'
) ;


require_once('./scripts/fasdim_include.inc') ;


/* ======================= Parameters ============================= */

$rep_courriers_doc =  $fasdim_config['doc_nonano'] ;
$rep_courriers_txt_nonano =  $fasdim_config['txt_nonano'] ;
$rep_courriers_txt_preano =  $fasdim_config['txt_preano'] ;
$rep_courriers_txt_ano =  $fasdim_config['txt_ano'] ; ;
$bdd_dsn = $fasdim_config['database_dsn']  ;
$bdd_user = $fasdim_config['database_user']  ; ;
$bdd_pwd = $fasdim_config['database_password'] ;
$nom_table = $fasdim_config['table_docnonano2txtnonano'] ;

$restrictions=FALSE ; // set this to FALSE if you want to bypass the restrictions we had to implement here
$limites_iep = array(6055,7000) ;  // Example of restriction: here we want to process only the files with a certain name... Only an example, don't worry.

$extensions = array('doc') ;

/* ======================= Main ============================= */

if( !is_dir($rep_courriers_doc)) {
	trigger_error( "The directory $rep_courriers_doc must exist and contain all the DOC files", E_USER_ERROR) ;
}
if( !is_dir($rep_courriers_txt_nonano)) {
	mkdir($rep_courriers_txt_nonano) ;
	trigger_error( "Directory creation: $rep_courriers_txt_nonano", E_USER_NOTICE) ;
}
if( !is_dir($rep_courriers_txt_preano)) {
	mkdir($rep_courriers_txt_preano) ;
	trigger_error( "Directory creation: $rep_courriers_txt_preano", E_USER_NOTICE) ;
}
if( !is_dir($rep_courriers_txt_ano)) {
	mkdir($rep_courriers_txt_ano) ;
	trigger_error( "Directory creation: $rep_courriers_txt_ano", E_USER_NOTICE) ;
}




// phase 2
print "\nListing of the DOC files" ;
if( $restrictions AND isset( $limites_iep ) AND count($limites_iep)>0) {
	print "\n\tBEWARE : only the files with an ID between ".implode(" and ",$limites_iep)." will be processed." ;
}


try {
    $bdd = new PDO(  $bdd_dsn, $bdd_user, $bdd_pwd ) ;
} catch (PDOException $e) {
    trigger_error('Connection failed: ' . $e->getMessage(), E_USER_ERROR) ;
}
$bdd->exec("CREATE TABLE $nom_table ( nom varchar(255) not null, statut char(1) not null, primary key (nom) )  ;");
$stt = $bdd->prepare("INSERT INTO $nom_table (nom, statut) VALUES (:nom, :statut) ;") ;


$fichiers_doc = glob( $rep_courriers_doc ."/*.*" ) ;
$nb_fichiers = count($fichiers_doc) ;

$barre = new barre_progression_batch(0, $nb_fichiers ) ;
$barre->dessine() ;
$i=0 ;

$nb_indexes=0 ;
$nb_ignores=0 ;
$iep_rencontres=array() ;
$ext_rencontrees=array() ;

foreach( $fichiers_doc as $un_fichier_doc) {
	$un_fichier_doc = basename($un_fichier_doc) ;
	$select = TRUE ;
	// here is an example of restriction management
	if( $restrictions ) {
		if( isset( $limites_iep ) and count($limites_iep)>0 ) {
			$tab = explode('-', $un_fichier_doc) ;
			$iep_str = substr($tab[1],0,4) ;
			$iep = (int) $iep_str ;
			if( !isset($iep_rencontres[$iep_str])) {
				$iep_rencontres[$iep_str]=0 ;
			}
			$iep_rencontres[$iep_str]++ ;
			if( $iep< $limites_iep[0] OR $iep> $limites_iep[1] )  {
				$select = FALSE ;
			}
		}
	}
	// end of the restrictions...
	$tab = explode('.', $un_fichier_doc) ;
	$ext = $tab[(count($tab)-1)] ;
	if( !isset($ext_rencontrees[$ext])) {
		$ext_rencontrees[$ext]=0 ;
	}
	$ext_rencontrees[$ext]++ ;
	if( !in_array( $ext, $extensions) ) {
		$select=FALSE ;
	}

	if($select) {
		$stt->execute(array(":nom"=>$un_fichier_doc, ":statut"=>0)) ;
		$nb_indexes++ ;
	} else {
		$nb_ignores++ ;
	}
	$i++ ;
	$barre->avance($i) ;
}

print "\nIndexed files: $nb_indexes" ;
print "\nIgnored files: $nb_ignores" ;
// here is an example of report based on the restrictions you can add in the script
if( $restrictions and isset( $limites_iep ) and count($limites_iep)>0 ) {
	print "\nIdentifiers met:\n" ;
	print_r($iep_rencontres) ;	
}
// end of the example
print "\nFile extensions:\n" ;
print_r($ext_rencontrees) ;

?>


STEP 01 DONE !!
Check the error log file if necessary.


