<?php

/*

    FASDIM: Fast And Simple DeIdentification Method
    Copyright (C) 2013 Emmanuel Chazard http://emmanuel.chazard.org

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/

print("
STEP 04
This program prepares TXT files for the de-identification:
- simplified typography
- suppression of salutation patterns
- suppression of date patterns
- suppression of patient names (if known)
If necessary, several instances of the program can be launched in the same time.
") ;

$liste_param=array( 'txt_nonano', 'txt_preano',
	'database_dsn', 'database_user', 'database_password',
	'table_txtnonano2txtpreano',
	'file_reports', 'file_patients_names'
) ;


require_once('./scripts/fasdim_include.inc') ;


/* ======================= Parameters ============================= */

$rep_entree =  $fasdim_config['txt_nonano'] ;
$rep_sortie =  $fasdim_config['txt_preano'] ;
$file_names = $fasdim_config['file_patients_names'] ;
$file_reports = $fasdim_config['file_reports'] ;

$bdd_dsn = $fasdim_config['database_dsn']  ;
$bdd_user = $fasdim_config['database_user']  ; ;
$bdd_pwd = $fasdim_config['database_password'] ;
$nom_table = $fasdim_config['table_txtnonano2txtpreano'] ;

/* ======================= Functions ============================= */


function purifier_texte($content) {
	$content = trim($content) ;
	
	// traitement particulier de l'apostrophe par WVWare
	$content = str_replace( chr(25), ' ', $content);
	
	// minuscules
	$content = strtolower( $content ) ;

	// dsaccentuation et certains caractres spciaux
	$content = str_replace( array('', '', ''), 'a', $content);
	$content = str_replace( array('', '', '', ''), 'e', $content);
	$content = str_replace( array( '', ''), 'i', $content);
	$content = str_replace( array( '', ''), 'o', $content);
	$content = str_replace( array( '', '', ''), 'u', $content);
	$content = str_replace( '', 'y', $content);
	$content = str_replace('', 'oe', $content) ;
	$content = str_replace('', 'ae', $content) ;
	$content = str_replace('', 'c', $content) ;

	// les chiffres
	// $content = preg_replace('|\d+|', '@', $content) ;

	// les monsieur madame... prnom et nom
	$content = preg_replace(array(
		'|\bmonsieur\s+\w+\s+\w+\b|',
		'|\bm\.\s+\w+\s+\w+\b|',
		'|\bmr\.\s+\w+\s+\w+\b|',
		'|\bmr\s+\w+\s+\w+\b|',
		'|\bmadame\s+\w+\s+\w+\b|',
		'|\bmme\.\s+\w+\s+\w+\b|',
		'|\bmme\s+\w+\s+\w+\b|',
		'|\bmademoiselle\s+\w+\s+\w+\b|',
		'|\bmelle\.\s+\w+\s+\w+\b|',
		'|\bmelle\s+\w+\s+\w+\b|',
		'|\bdocteur\s+\w+\s+\w+\b|',
		'|\bdr\.\s+\w+\s+\w+\b|',
		'|\bdr\s+\w+\s+\w+\b|',
		'|\bprofesseur\s+\w+\s+\w+\b|',
		'|\bpr\.\s+\w+\s+\w+\b|',
		'|\bpr\s+\w+\s+\w+\b|',
	), '@ @ @', $content) ;

	// les monsieur madame... initiale du prnom, point, et nom
	$content = preg_replace(array(
		'|\bmonsieur\s+\w\.\s?\w+\b|',
		'|\bm\.\s+\w\.\s?\w+\b|',
		'|\bmr\.\s+\w\.\s?\w+\b|',
		'|\bmr\s+\w\.\s?\w+\b|',
		'|\bmadame\s+\w\.\s?\w+\b|',
		'|\bmme\.\s+\w\.\s?\w+\b|',
		'|\bmme\s+\w\.\s?\w+\b|',
		'|\bmademoiselle\s+\w\.\s?\w+\b|',
		'|\bmelle\.\s+\w\.\s?\w+\b|',
		'|\bmelle\s+\w\.\s?\w+\b|',
		'|\bdocteur\s+\w\.\s?\w+\b|',
		'|\bdr\.\s+\w\.\s+\w?\b|',
		'|\bdr\s+\w\.\s+\w?\b|',
		'|\bprofesseur\s+\w\.\s?\w+\b|',
		'|\bpr\.\s+\w\.\s?\w+\b|',
		'|\bpr\s+\w\.\s?\w+\b|',
	), '@ @ @', $content) ;

	// les monsieur madame... nom seulement
	$content = preg_replace(array(
		'|\bmonsieur\s+\w+\b|',
		'|\bm\.\s+\w+\b|',
		'|\bmr\.\s+\w+\b|',
		'|\bmr\s+\w+\b|',
		'|\bmadame\s+\w+\b|',
		'|\bmme\.\s+\w+\b|',
		'|\bmme\s+\w+\b|',
		'|\bmademoiselle\s+\w+\b|',
		'|\bmelle\.\s+\w+\b|',
		'|\bmelle\s+\w+\b|',
		'|\bdocteur\s+\w+\b|',
		'|\bdr\.\s+\w+\b|',
		'|\bdr\s+\w+\b|',
		'|\bprofesseur\s+\w+\b|',
		'|\bpr\.\s+\w+\b|',
		'|\bpr\s+\w+\b|',
	), '@ @', $content) ;

	// certains lments de dates
	$content = preg_replace(array(
		'|\blundi\b|',
		'|\bmardi\b|',
		'|\bmercredi\b|',
		'|\bjeudi\b|',
		'|\bvendredi\b|',
		'|\bsamedi\b|',
		'|\bdimanche\b|',
		'|\bjanvier\b|',
		'|\bfevrier\b|',
		'|\bmars\b|',
		'|\bavril\b|',
		'|\bmai\b|',
		'|\bjuin\b|',
		'|\bjuillet\b|',
		'|\baout\b|',
		'|\bseptembre\b|',
		'|\boctobre\b|',
		'|\bnovembre\b|',
		'|\bdecembre\b|'
	), '@', $content) ;


	// $content = str_replace( "\r", '', $content);

	return $content ;
}



/*  =========== Pqatients names loading =========== */

if( !is_file( $file_names ) or !is_file( $file_reports )) {

	print "\n\nWARNING: patient names won't be suppressed, because the related files are not available.\n\n" ;
	$supprimer_noms_patients = FALSE ;

} else {

	$supprimer_noms_patients = TRUE ;

	// retrouver l'ID patient depuis le nom du courrier
	$memoire_courriers = array() ;

	$file = $file_reports ;
	$fp = fopen( $file, 'r' ) ;
	if( !$fp ) {
		trigger_error( "The file $file cannot be found", E_USER_ERROR ) ;
	}
	while( !feof( $fp ) ) {
		$line = fgets( $fp ) ;
		$cells = explode("\t", $line) ;
		if( count($cells)>3) {
			$id_patient = ltrim( trim($cells[2]), ' 0') ; ;
			$id_courrier = $cells[4] ;
			$id_courrier = str_replace('.doc', '.txt', trim($id_courrier) ) ;  /* ... as the files have been renamed previously... */
			$memoire_courriers[ $id_courrier ] = $id_patient ;
			// print "$id_courrier \t $id_patient \n" ;
		}
	}
	fclose( $fp ) ;


	// les noms  supprimer pour un patient donn
	$memoire_mots = array() ;

	$file = $file_names ;
	$fp = fopen( $file, 'r' ) ;
	if( !$fp ) {
		trigger_error( "fichier introuvable $file", E_USER_ERROR ) ;
	}
	while( ! feof( $fp ) ) {
		$line = fgets( $fp ) ;
		$cells = explode("\t", $line) ;
		if( count($cells)>4) {
			$id_patient = ltrim( $cells[1], ' 0') ; ;
			$memoire_mots[ $id_patient ] = array() ;
			$nom = $cells[2] ;
			if( strlen($nom)>2 ) {
				$memoire_mots[ $id_patient ][] = $nom ;
				if( strstr($nom, '-')) {
					$temp = explode('-', $nom) ;
					foreach( $temp as $value ) {
						$memoire_mots[ $id_patient ][] = $value ;
					}
				}
			}
			$nom = $cells[3] ;
			if( strlen($nom)>2 ) {
				$memoire_mots[ $id_patient ][] = $nom ;
				if( strstr($nom, '-')) {
					$temp = explode('-', $nom) ;
					foreach( $temp as $value ) {
						$memoire_mots[ $id_patient ][] = $value ;
					}
				}
			}
			$nom = $cells[4] ;
			if( strlen($nom)>2 ) {
				$memoire_mots[ $id_patient ][] = $nom ;
				if( strstr($nom, '-')) {
					$temp = explode('-', $nom) ;
					foreach( $temp as $value ) {
						$memoire_mots[ $id_patient ][] = $value ;
					}
				}
			}
			/* Birth date: useless
			$nom = $cells[5] ;
			if( strlen($nom)>2 ) {
				$memoire_mots[ $id_patient ][] = $nom ;
			}
			*/
		}
	}
	fclose( $fp ) ;


}






/*  =========== MAIN =========== */

try {
	$bdd = new PDO(  $bdd_dsn, $bdd_user, $bdd_pwd ) ;
} catch (PDOException $e) {
	trigger_error('Connection failed: ' . $e->getMessage(), E_USER_ERROR) ;
}


print( bdd_etat_verrous($bdd, $nom_table) );
$i=0 ;
print "\n\n\n$i records" ;


while( $file = bdd_fichier_verrouille($bdd, $nom_table) ) {
	$un_file_in = $rep_entree.'/'. $file ;
	$un_file_out = $rep_sortie.'/'. $file ;
	$content = file_get_contents( $un_file_in, FILE_TEXT ) ;
	// purification typographique et suppression aveugle de certains motifs
	$content = purifier_texte($content) ;

	// si on les a, on supprime d'abord les noms des personnes
	if( $supprimer_noms_patients  ) {
		$mots = array() ;

		if( isset( $memoire_courriers[$file] ) ) {
			$id_patient = $memoire_courriers[$file] ;
			if( isset( $memoire_mots[ $id_patient ] ) and count($memoire_mots[ $id_patient ])>0) {
				$mots = $memoire_mots[ $id_patient ] ;
			} else {
				trigger_error("The record $file couldn't be de-identified (undefined words)", E_USER_NOTICE) ;
			}
		} else {
			trigger_error("The record $file couldn't be de-identified (unknown patient)", E_USER_NOTICE) ;
		}

		foreach( $mots as $un_mot ) {
			if( !strstr($un_mot, '.' ) AND !strstr($un_mot, '+' ) AND !strstr($un_mot, '*' ) AND !strstr($un_mot, '?') ) {
				$content = preg_replace("|\b$un_mot\b|iU", '@', $content )  ;
			}
		}
	}

	// finalement on enregistre dans le nouveau fichier
	file_put_contents( $un_file_out, $content, FILE_TEXT ) ;
	bdd_fichier_deverrouille($bdd, $nom_table, $file) ;
	$i++ ;
    print "\r$i courriers" ;
}


print( "\n\n". bdd_etat_verrous($bdd, $nom_table) . "\n\n" );



?>



STEP 04 DONE
Please check the error log file if necessary.


